<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:template="http://www.boddie.org.uk/ns/xmltools/template">

  <xsl:output indent="yes"/>

  <xsl:template match="*[@template:attribute-field]">
    <xsl:copy>
      <!-- Remove attribute-field and replace name and value. -->
      <xsl:apply-templates select="@*[local-name() != 'attribute-field' and local-name() != 'name' and local-name() != 'value']"/>
      <xsl:attribute name="template:attribute"><xsl:value-of select="@template:attribute-field"/></xsl:attribute>
      <xsl:attribute name="name">{template:this-attribute()}</xsl:attribute>
      <xsl:attribute name="value">{$this-value}</xsl:attribute>
      <xsl:apply-templates select="*|node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="*[@template:selector-field]">
    <xsl:variable name="field-name" select="@template:selector-field"/>
    <xsl:copy>
      <!-- Remove selector-field and replace name. -->
      <xsl:apply-templates select="@*[local-name() != 'selector-field' and local-name() != 'name']"/>
      <xsl:attribute name="name"><xsl:value-of select="$field-name"/>={template:this-element()}</xsl:attribute>
      <xsl:apply-templates select="*|node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="*[@template:multiple-choice-field]">
    <xsl:variable name="field-element" select="substring-before(@template:multiple-choice-field, ',')"/>
    <xsl:variable name="field-attr" select="substring-after(@template:multiple-choice-field, ',')"/>
    <xsl:copy>
      <!-- Remove multiple-choice-field and replace name. -->
      <xsl:apply-templates select="@*[local-name() != 'multiple-choice-field' and local-name() != 'name']"/>
      <xsl:attribute name="template:element"><xsl:value-of select="$field-element"/></xsl:attribute>
      <xsl:attribute name="name">{template:new-attribute('<xsl:value-of select="$field-attr"/>')}</xsl:attribute>
      <xsl:apply-templates select="*|node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="*[@template:multiple-choice-value]">
    <xsl:variable name="field-element" select="substring-before(@template:multiple-choice-value, ',')"/>
    <xsl:variable name="field-attr-info" select="substring-after(@template:multiple-choice-value, ',')"/>
    <xsl:variable name="field-attr" select="substring-before($field-attr-info, ',')"/>
    <xsl:variable name="field-set-attr" select="substring-after($field-attr-info, ',')"/>
    <xsl:copy>
      <!-- Remove multiple-choice-value and replace value. -->
      <xsl:apply-templates select="@*[local-name() != 'multiple-choice-value' and local-name() != 'value']"/>
      <xsl:attribute name="template:element"><xsl:value-of select="$field-element"/></xsl:attribute>
      <xsl:attribute name="template:expr">@<xsl:value-of select="$field-attr"/> = ../@<xsl:value-of select="$field-attr"/></xsl:attribute>
      <xsl:attribute name="template:expr-attr"><xsl:value-of select="$field-set-attr"/></xsl:attribute>
      <xsl:attribute name="template:value">@<xsl:value-of select="$field-attr"/></xsl:attribute>
      <xsl:attribute name="value">{@<xsl:value-of select="$field-attr"/>}</xsl:attribute>
      <xsl:apply-templates select="*|node()"/>
    </xsl:copy>
  </xsl:template>

  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
